==> haystack-0.5.0%2B2.tar.gz: lib/src/HCoord.dart <==
///
/// Copyright (c) 2018, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.hcoord;

import 'package:intl/intl.dart';
import 'package:quiver/core.dart';

import 'HVal.dart';
import 'io/HZincReader.dart';
import 'str-utils.dart';

/// HBool defines singletons for true/false tag values.
///
/// See [Project Haystack](http://project-haystack.org/doc/TagModel#tagKinds)
class HCoord extends HVal {

  /// Construct from basic fields 
  factory HCoord.from( Object o, [num lng]) {
    if ( o is String ) return new HCoord.fromString(o);
    if ( o is num ) return new HCoord(o, lng);
    return null;
  }

  factory HCoord(num lat, num lng) {
    int ulat = (lat * 1000000.0).toInt();
    int ulng = (lng * 1000000.0).toInt();

    return new HCoord._(ulat, ulng);
  }

  /// Parse from string fomat "C(lat,lng)" or raise ParseException
  factory HCoord.fromString(String s) {
    return new HZincReader(s).readVal() as HCoord;
  }

  HCoord._(this.ulat, this.ulng) {
    if (ulat < -90000000 || ulat > 90000000) throw new ArgumentError("Invalid lat > +/- 90");
    if (ulng < -180000000 || ulng > 180000000) throw new ArgumentError("Invalid lng > +/- 180");
  }

  /// Return if given latitude is legal value between -90.0 and +90.0 
  static bool isLat(double lat) => -90.0 <= lat && lat <= 90.0;

  /// Return if given is longtitude is legal value between -180.0 and +180.0 
  static bool isLng(double lng) => -180.0 <= lng && lng <= 180.0;

  /// Latitude in decimal degrees 
  double lat() => ulat / 1000000.0;

  /// Longtitude in decimal degrees 
  double lng() => ulng / 1000000.0;

  /// Latitude in micro-degrees 
  final int ulat;

  /// Longitude in micro-degrees 
  final int ulng;

  /// Hash is based on lat/lng 
  @override
  int get hashCode {
    if (_hashCode == null) {
      _hashCode = hash2(ulat, ulng);
    }
    return _hashCode;
  }
  int _hashCode;

  /// Equality is based on lat/lng 
  @override
  bool operator ==(Object that) => that is HCoord && ulat == that.ulat && ulng == that.ulng;

  /// Return "c:lat,lng" 
  @override
  String toJson() => (new StringBuffer()
      ..write("c:")
      ..write(_uToStr(ulat))
      ..write(',')
      ..write(_uToStr(ulng))).toString();

  /// Represented as "C(lat,lng)" 
  @override
  String toZinc() {
    StringBuffer s = new StringBuffer()
      ..writeAll( <String>["C(", _uToStr(ulat), ',', _uToStr(ulng), ")" ], "" );
    return s.toString();
  }

  static String _uToStr(int ud) {
    StringBuffer s = new StringBuffer();

    if (ud < 0) { s.writeCharCode(MINUS_CHAR); ud = -ud; }

    if (ud < 1000000.0) {
      s.write(new NumberFormat("0.0#####", 'en_US').format(ud/1000000.0));
    } else {
      String x = ud.toString();
      int dot = x.length - 6;
      int end = x.length;
      while (end > dot+1 && x.codeUnitAt(end-1) == ZERO_CHAR) --end;
      for (int i=0; i<dot; ++i) s.writeCharCode(x.codeUnitAt(i));
      s.writeCharCode(PERIOD_CHAR);
      for (int i=dot; i<end; ++i) s.writeCharCode(x.codeUnitAt(i));
    }

    return s.toString();
  }
}

==> haystack-0.6.0-dev.0.1%2B2.tar.gz: lib/src/HCoord.dart <==
///
/// Copyright (c) 2018, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.hcoord;

import 'package:intl/intl.dart';
import 'package:quiver/core.dart';

import 'HVal.dart';
import 'io/HZincReader.dart';
import 'str-utils.dart';

/// HBool defines singletons for true/false tag values.
///
/// See [Project Haystack](http://project-haystack.org/doc/TagModel#tagKinds)
class HCoord extends HVal {
  /// Construct from basic fields
  factory HCoord.from(Object o, [num lng]) {
    if (o is String) return new HCoord.fromString(o);
    if (o is num) return new HCoord(o, lng);
    return null;
  }

  factory HCoord(num lat, num lng) {
    int ulat = (lat * 1000000.0).toInt();
    int ulng = (lng * 1000000.0).toInt();

    return new HCoord._(ulat, ulng);
  }

  /// Parse from string fomat "C(lat,lng)" or raise ParseException
  factory HCoord.fromString(String s) {
    return new HZincReader(s).readVal() as HCoord;
  }

  HCoord._(this.ulat, this.ulng) {
    if (ulat < -90000000 || ulat > 90000000)
      throw new ArgumentError("Invalid lat > +/- 90");
    if (ulng < -180000000 || ulng > 180000000)
      throw new ArgumentError("Invalid lng > +/- 180");
  }

  /// Return if given latitude is legal value between -90.0 and +90.0
  static bool isLat(double lat) => -90.0 <= lat && lat <= 90.0;

  /// Return if given is longtitude is legal value between -180.0 and +180.0
  static bool isLng(double lng) => -180.0 <= lng && lng <= 180.0;

  /// Latitude in decimal degrees
  double lat() => ulat / 1000000.0;

  /// Longtitude in decimal degrees
  double lng() => ulng / 1000000.0;

  /// Latitude in micro-degrees
  final int ulat;

  /// Longitude in micro-degrees
  final int ulng;

  /// Hash is based on lat/lng
  @override
  int get hashCode {
    if (_hashCode == null) {
      _hashCode = hash2(ulat, ulng);
    }
    return _hashCode;
  }

  int _hashCode;

  /// Equality is based on lat/lng
  @override
  bool operator ==(Object that) =>
      that is HCoord && ulat == that.ulat && ulng == that.ulng;

  /// Return "c:lat,lng"
  @override
  String toJson() => (new StringBuffer()
        ..write("c:")
        ..write(_uToStr(ulat))
        ..write(',')
        ..write(_uToStr(ulng)))
      .toString();

  /// Represented as "C(lat,lng)"
  @override
  String toZinc() {
    StringBuffer s = new StringBuffer()
      ..writeAll(<String>["C(", _uToStr(ulat), ',', _uToStr(ulng), ")"], "");
    return s.toString();
  }

  static String _uToStr(int ud) {
    StringBuffer s = new StringBuffer();

    if (ud < 0) {
      s.writeCharCode(MINUS_CHAR);
      ud = -ud;
    }

    if (ud < 1000000.0) {
      s.write(new NumberFormat("0.0#####", 'en_US').format(ud / 1000000.0));
    } else {
      String x = ud.toString();
      int dot = x.length - 6;
      int end = x.length;
      while (end > dot + 1 && x.codeUnitAt(end - 1) == ZERO_CHAR) --end;
      for (int i = 0; i < dot; ++i) s.writeCharCode(x.codeUnitAt(i));
      s.writeCharCode(PERIOD_CHAR);
      for (int i = dot; i < end; ++i) s.writeCharCode(x.codeUnitAt(i));
    }

    return s.toString();
  }
}
